package dk.nsp.idsas.kafka;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import dk.nsp.idsas.dao.entity.BlurringEntity;
import dk.nsp.idsas.dao.entity.OrgBlurringEntity;
import dk.nsp.idsas.dto.KafkaMsgType;
import dk.nsp.idsas.service.IdsasServiceException;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.FaultCode;
import org.apache.kafka.clients.producer.Producer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.clients.producer.RecordMetadata;
import org.apache.log4j.Logger;

import java.nio.charset.StandardCharsets;
import java.util.UUID;
import java.util.concurrent.Future;

public class KafkaProducerService {
    public static final String KAFKA_CLIENT_ID_KEY = "client.id";

    private static final Logger logger = Logger.getLogger(KafkaProducerService.class);

    private final String topic;
    private final Producer<String, String> kafkaProducer;

    private final ObjectMapper mapper = new ObjectMapper();

    public KafkaProducerService(String topic, Producer<String, String> kafkaProducer) {
        this.topic = topic;
        this.kafkaProducer = kafkaProducer;

        // Så vi kan få serialiseret vores Instants som ISO datoer
        mapper.disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS);
        mapper.registerModule(new JavaTimeModule());
    }

    public void sendCreateBlurringMessage(BlurringEntity entity) throws IdsasServiceException {
        sendMessage(entity, KafkaMsgType.CREATE_BLURRING_ENTITY.getValue());
    }

    public void sendCreateOrgBlurringMessage(OrgBlurringEntity entity) throws IdsasServiceException {
        sendMessage(entity, KafkaMsgType.CREATE_ORG_BLURRING_ENTITY.getValue());
    }

    public void removeOrgBlurringMessage(OrgBlurringEntity entity) throws IdsasServiceException {
        sendMessage(entity, KafkaMsgType.REMOVE_ORG_BLURRING_ENTITY.getValue());
    }

    private void sendMessage(Object o, String typeName) throws IdsasServiceException {
        try {
            String key = UUID.randomUUID().toString();
            String value = mapper.writeValueAsString(o);
            ProducerRecord<String, String> producerRecord = new ProducerRecord<>(topic, key, value);
            producerRecord.headers().add("type", typeName.getBytes(StandardCharsets.UTF_8));

            logger.info(String.format("Sending Kafka message. Key: %s, Value: %s", key, value));
            Future<RecordMetadata> recordMetadataFuture = kafkaProducer.send(producerRecord);
            RecordMetadata recordMetadata = recordMetadataFuture.get();
            if (recordMetadata.topic() == null) {
                throw new Exception(ErrorMessages.unknownKafkaTopic());
            }
        } catch (Exception e) {
            String msg = "Fejl ved afsendelse af Kafka besked.";
            throw new IdsasServiceException(ErrorMessages.blurringError(msg), FaultCode.CREATE_BLURRING_ERROR, false, e);
        }
    }
}
